<?php

/**
 * @file
 * Contains \Drupal\paynl_payment\Entity\PaynlProfile\PaynlProfileForm.
 */

namespace Drupal\paynl_payment\Entity\PaynlProfile;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\plugin\Plugin\Plugin\PluginSelector\PluginSelectorManagerInterface;
use Drupal\plugin\PluginType\PluginTypeManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the Pay.nl profile add/edit form.
 */
class PaynlProfileForm extends EntityForm {

  /**
   * The paynl profile storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $paynlProfileStorage;

  /**
   * The plugin selector manager.
   *
   * @var \Drupal\plugin\Plugin\Plugin\PluginSelector\PluginSelectorManagerInterface
   */
  protected $pluginSelectorManager;

  /**
   * The plugin type manager.
   *
   * @var \Drupal\plugin\PluginType\PluginTypeManager
   */
  protected $pluginTypeManager;

  /**
   * Constructs a new instance.
   *
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translator.
   * @param \Drupal\Core\Entity\EntityStorageInterface $paynl_profile_storage
   *   The paynl profile storage.
   * @param \Drupal\plugin\Plugin\Plugin\PluginSelector\PluginSelectorManagerInterface $plugin_selector_manager
   *   The plugin selector manager.
   * @param \Drupal\plugin\PluginType\PluginTypeManager $plugin_type_manager
   *   The plugin type manager.
   */
  public function __construct(TranslationInterface $string_translation, EntityStorageInterface $paynl_profile_storage, PluginSelectorManagerInterface $plugin_selector_manager, PluginTypeManager $plugin_type_manager) {
    $this->paynlProfileStorage = $paynl_profile_storage;
    $this->pluginSelectorManager = $plugin_selector_manager;
    $this->pluginTypeManager = $plugin_type_manager;
    $this->stringTranslation = $string_translation;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /** @var \Drupal\Core\Entity\EntityManagerInterface $entity_manager */
    $entity_manager = $container->get('entity.manager');

    return new static($container->get('string_translation'), $entity_manager->getStorage('paynl_profile'), $container->get('plugin.manager.plugin.plugin_selector'), $container->get('plugin.plugin_type_manager'));
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\paynl_payment\Entity\PaynlProfilesInterface $paynl_profile */
    $paynl_profile = $this->getEntity();
    $form['label'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#default_value' => $paynl_profile->label(),
      '#maxlength' => 255,
      '#required' => TRUE,
    );
    $form['id'] = array(
      '#default_value' => $paynl_profile->id(),
      '#disabled' => !$paynl_profile->isNew(),
      '#machine_name' => array(
        'source' => array('label'),
        'exists' => array($this, 'PaynlProfileIdExists'),
      ),
      '#maxlength' => 255,
      '#type' => 'machine_name',
      '#required' => TRUE,
    );
    $form['apiKey'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Pay.nl API key'),
      '#default_value' => $paynl_profile->getApiKey(),
      '#maxlength' => 255,
      '#required' => TRUE,
    );
    $form['serviceId'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Pay.nl Service ID'),
      '#default_value' => $paynl_profile->getServiceId(),
      '#maxlength' => 255,
      '#required' => TRUE,
    );
    $form['testMode'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Testmode'),
      '#default_value' => $paynl_profile->hasTestMode(),
    );

    return parent::form($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function copyFormValuesToEntity(EntityInterface $paynl_profile, array $form, FormStateInterface $form_state) {
    /** @var \Drupal\paynl_payment\Entity\PaynlPaymentInterface $paynl_profile */
    parent::copyFormValuesToEntity($paynl_profile, $form, $form_state);
    $values = $form_state->getValues();
    $paynl_profile->setId($values['id']);
    $paynl_profile->setLabel($values['label']);
    $paynl_profile->setApiKey($values['apiKey']);
    $paynl_profile->setServiceId($values['serviceId']);
    $paynl_profile->setTestMode($values['testMode']);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $paynl_profile = $this->getEntity();
    $paynl_profile->save();
    drupal_set_message($this->t('@label has been saved.', array(
      '@label' => $paynl_profile->label()
    )));
    $form_state->setRedirect('entity.paynl_profile.collection');
  }

  /**
   * Checks if a Pay.nl profile with a particular ID already exists.
   *
   * @param string $id
   *
   * @return bool
   */
  public function paynlProfileIdExists($id) {
    return (bool) $this->paynlProfileStorage->load($id);
  }

}
