<?php

/**
 * @file
 * Contains \Drupal\paynl_payment\Plugin\Payment\Method\PaynlPayment.
 */

namespace Drupal\paynl_payment\Plugin\Payment\Method;

use Drupal;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\Core\Utility\Token;
use Drupal\payment\Entity\Payment;
use Drupal\payment\Entity\PaymentInterface;
use Drupal\payment\EventDispatcherInterface;
use Drupal\payment\Plugin\Payment\Status\PaymentStatusManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\payment\OperationResult;
use Drupal\payment\Plugin\Payment\Method\PaymentMethodBase;
use Drupal\payment\Plugin\Payment\Method\PaymentMethodCapturePaymentInterface;
use Drupal\payment\Plugin\Payment\Method\PaymentMethodRefundPaymentInterface;
use Drupal\payment\Plugin\Payment\Method\PaymentMethodUpdatePaymentStatusInterface;
use Drupal\payment\Response\Response;
use Drupal\paynl_payment\Entity\PaynlProfile;
use Paynl\Config;
use Paynl\Transaction;

/**
 * A payment method using Pay.nl.
 *
 * Plugins that extend this class must have the following keys in their plugin
 * definitions:
 * - entity_id: (string) The ID of the payment method entity the plugin is for.
 * - execute_status_id: (string) The ID of the payment status plugin to set at
 *   payment execution.
 * - capture: (boolean) Whether or not payment capture is supported.
 * - capture_status_id: (string) The ID of the payment status plugin to set at
 *   payment capture.
 * - refund: (boolean) Whether or not payment refunds are supported.
 * - refund_status_id: (string) The ID of the payment status plugin to set at
 *   payment refund.
 *
 * @PaymentMethod(
 *   deriver = "Drupal\paynl_payment\Plugin\Payment\Method\PaynlPaymentDeriver",
 *   id = "paynl_payment",
 *   operations_provider = "\Drupal\paynl_payment\Plugin\Payment\Method\PaynlPaymentOperationsProvider",
 * )
 */
class PaynlPayment extends PaymentMethodBase implements ContainerFactoryPluginInterface, PaymentMethodCapturePaymentInterface, PaymentMethodRefundPaymentInterface, PaymentMethodUpdatePaymentStatusInterface {

  /**
   * The payment status manager.
   *
   * @var \Drupal\payment\Plugin\Payment\Status\PaymentStatusManagerInterface
   */
  protected $paymentStatusManager;

  /**
   * Constructs a new instance.
   *
   * @param mixed[] $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed[] $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\payment\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   * @param \Drupal\Core\Utility\Token $token
   *   The token API.
   * @param \Drupal\payment\Plugin\Payment\Status\PaymentStatusManagerInterface
   *   The payment status manager.
   */
  public function __construct(array $configuration, $plugin_id, array $plugin_definition, ModuleHandlerInterface $module_handler, EventDispatcherInterface $event_dispatcher, Token $token, PaymentStatusManagerInterface $payment_status_manager) {
    $configuration += $this->defaultConfiguration();
    parent::__construct($configuration, $plugin_id, $plugin_definition, $module_handler, $event_dispatcher, $token, $payment_status_manager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition, $container->get('module_handler'), $container->get('payment.event_dispatcher'), $container->get('token'), $container->get('plugin.manager.payment.status'));
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedCurrencies() {
    return TRUE;
  }

  /**
   * Gets the ID of the payment method this plugin is for.
   *
   * @return string
   */
  public function getEntityId() {
    return $this->pluginDefinition['entity_id'];
  }

  /**
   * Gets the status to set on payment execution.
   *
   * @return string
   *   The plugin ID of the payment status to set.
   */
  public function getExecuteStatusId() {
    return $this->pluginDefinition['execute_status_id'];
  }

  /**
   * Gets the status to set on payment capture.
   *
   * @return string
   *   The plugin ID of the payment status to set.
   */
  public function getCaptureStatusId() {
    return $this->pluginDefinition['capture_status_id'];
  }

  /**
   * Gets whether or not capture is supported.
   *
   * @param bool
   *   Whether or not to support capture.
   */
  public function getCapture() {
    return $this->pluginDefinition['capture'];
  }

  /**
   * Gets the status to set on payment refund.
   *
   * @return string
   *   The plugin ID of the payment status to set.
   */
  public function getRefundStatusId() {
    return $this->pluginDefinition['refund_status_id'];
  }

  /**
   * Gets whether or not capture is supported.
   *
   * @param bool
   *   Whether or not to support capture.
   */
  public function getRefund() {
    return $this->pluginDefinition['refund'];
  }

  /**
   * Get the Pay.nl profile to use.
   *
   * @param bool
   *   Whether or not to support capture.
   */
  public function getProfile() {
    $profile_id = $this->pluginDefinition['profile'];
    $profile = PaynlProfile::load($profile_id);

    return $profile;
  }

  /**
   * {@inheritdoc}
   */
  public function getPaymentExecutionResult() {
    // Get the paynl profile.
    $paynl_profile = $this->getProfile();

    try {
      // Configure API.
      $api_key = $paynl_profile->getApiKey();
      $service_id = $paynl_profile->getServiceId();
      Config::setApiToken($api_key);
      Config::setServiceId($service_id);

      try {
        $payment = $this->getPayment();

        // Get line items to pass them to Pay.nl.
        $line_items = array();
        foreach ($payment->getLineItems() as $line_item) {
          $line_items[] = array(
            'id' => $line_item->getName(),
            'name' => $line_item->getName(),
            'price' => $line_item->unitAmount(TRUE),
            'tax' => $line_item->unitAmount(TRUE) - $line_item->unitAmount(FALSE),
            'qty' => $line_item->getQuantity(),
          );
        }

        try {
          // Start transaction.
          $transaction = Transaction::start(array(
            'amount' => $payment->getAmount(),
            'returnUrl' => Drupal::url('paynl_payment.return',
                array('payment' => $payment->id()), array('absolute' => TRUE)),
            'exchangeUrl' => Drupal::url('paynl_payment.exchange',
                array('payment' => $payment->id()), array('absolute' => TRUE)),
//            'paymentMethod' => $payment->context_data['payment_method'],
            'description' => $payment->id(),
            'products' => $line_items,
            'language' => strtoupper(Drupal::languageManager()->getCurrentLanguage()->getId()),
            'ipaddress' => Drupal::request()->getClientIp(),
            'testmode' => $paynl_profile->hasTestMode(),
          ));

          // Redirect to PSP.
          $redirect_url = $transaction->getRedirectUrl();

          $url = Url::fromUri($redirect_url);
          $response = new Response($url);
          return new OperationResult($response);
        }
        catch(\Exception $e) {
          drupal_set_message($e->getMessage(), 'error');
          return;
        }
      }
      catch (\Exception $e) {
        drupal_set_message($e->getMessage(), 'error');
        return;
      }
    }
    catch (\Exception $e) {
      drupal_set_message($e->getMessage(), 'error');
      return;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function doCapturePayment() {
    $this->getPayment()->setPaymentStatus($this->paymentStatusManager->createInstance($this->getCaptureStatusId()));
    $this->getPayment()->save();
  }

  /**
   * {@inheritdoc}
   */
  public function doCapturePaymentAccess(AccountInterface $account) {
    return $this->getCapture() && $this->getPayment()->getPaymentStatus()->getPluginId() != $this->getCaptureStatusId();
  }

  /**
   * {@inheritdoc}
   */
  public function doRefundPayment() {
    $this->getPayment()->setPaymentStatus($this->paymentStatusManager->createInstance($this->getRefundStatusId()));
    $this->getPayment()->save();
  }

  /**
   * {@inheritdoc}
   */
  public function doRefundPaymentAccess(AccountInterface $account) {
    return $this->getRefund() && $this->getPayment()->getPaymentStatus()->getPluginId() != $this->getRefundStatusId();
  }

  /**
   * {@inheritdoc}
   */
  public function updatePaymentStatusAccess(AccountInterface $account) {
    return AccessResult::forbidden();
  }

  /**
   * {@inheritdoc}
   */
  public function getSettablePaymentStatuses(AccountInterface $account, PaymentInterface $payment) {
    return [];
  }

  /**
   * Update payment status,
   */
  public function updatePaymentStatus(Payment $payment, $payment_status) {
    $payment->setPaymentStatus($payment->getPaymentMethod()->paymentStatusManager->createInstance($payment_status));
    $payment->save();
  }

}
